import * as mc from "@minecraft/server";
import { ItemStack } from "@minecraft/server";
import { paintbrushColors, stateMapping, BlockMapping, extractColor, VanillaRevertMapping } from "./constants.js";
itemUseListener();
function itemUseListener() {
  mc.world.afterEvents.itemUse.subscribe(handleItemUse);
  mc.world.afterEvents.projectileHitBlock.subscribe(handleProjectileHitBlock);
}
function handleItemUse(eventData) {
  const player = eventData.source;
  const itemUse = eventData.itemStack;
  if (!itemUse.typeId.startsWith("ascent_paint:") || itemUse.typeId.startsWith("ascent_paint:") && itemUse.typeId.includes("paintball")) {
    return;
  }
  const hitBlock = getHitBlock(player);
  if (!hitBlock) {
    return sendDeveloperMessage(player, "\xA74\xA7l[!]\xA7c\xA7l Paint Error:\xA7r\xA77 No block in view.\xA7r", "note.bassattack");
  }
  if (!(hitBlock.typeId.startsWith("ascent_paint:") || hitBlock.typeId.startsWith("minecraft:"))) {
    return;
  }
  if (itemUse.typeId === "ascent_paint:cleaning_sponge") {
    if (tryRevertBlock(player, hitBlock)) {
      handleToolDurability(player, itemUse);
      player.playSound("ascent_paint:cleaning_sponge.clean");
      mc.world.scoreboard.getObjective("ascent_paint:jig_computer.addon_stats")?.addScore("ascent_paint:blocks_cleaned", 1);
    }
  } else {
    const color = getColorFromItem(itemUse.typeId);
    if (!color) {
      return sendDeveloperMessage(player, `\xA74\xA7l[!]\xA7c\xA7l Paint Error:\xA7r\xA77 Item ${itemUse.typeId} is not a recognized paintbrush.\xA7r`, "note.bassattack");
    }
    if (tryReplaceBlock(player, hitBlock, color)) {
      const emptyPaintbrush = handleToolDurability(player, itemUse);
      if (emptyPaintbrush)
        return player.playSound("ascent_paint:paintbrush.paint", { pitch: 1.5 });
      player.playSound("ascent_paint:paintbrush.paint");
      mc.world.scoreboard.getObjective("ascent_paint:jig_computer.addon_stats")?.addScore("ascent_paint:blocks_painted", 1);
    }
  }
}
function handleProjectileHitBlock(eventData) {
  const projectile = eventData.projectile;
  const location = eventData.location;
  const dim = eventData.dimension;
  const player = eventData.source;
  const color = getColorFromItem(projectile.typeId);
  if (!color && projectile.typeId !== "minecraft:splash_potion")
    return;
  if ((!projectile.typeId.startsWith("ascent_paint:") || !projectile.typeId.includes("paintball")) && projectile.typeId !== "minecraft:splash_potion") {
    return;
  }
  const startBlock = dim.getBlock(location);
  if (startBlock === void 0)
    return;
  const loc = startBlock.center();
  const range = 2;
  const validBlocks = [];
  for (let x = loc.x - range; x <= loc.x + range; x++) {
    for (let y = loc.y - range; y <= loc.y + range; y++) {
      for (let z = loc.z - range; z <= loc.z + range; z++) {
        const distance = (x - loc.x) ** 2 + (y - loc.y) ** 2 + (z - loc.z) ** 2;
        if (distance > range ** 2)
          continue;
        const currentBlock = dim.getBlock({ x, y, z });
        if (currentBlock && !currentBlock.isAir && !currentBlock.isLiquid && currentBlock.isValid()) {
          validBlocks.push(currentBlock);
        }
      }
    }
  }
  for (let block of validBlocks) {
    if (projectile.typeId === "minecraft:splash_potion") {
      if (tryRevertBlock(player, block)) {
        mc.world.scoreboard.getObjective("ascent_paint:jig_computer.addon_stats")?.addScore("ascent_paint:blocks_cleaned", 1);
      }
    }
    if (tryReplaceBlock(player, block, color)) {
      mc.world.playSound("ascent_paint:paintbrush.paint", startBlock);
      mc.world.scoreboard.getObjective("ascent_paint:jig_computer.addon_stats")?.addScore("ascent_paint:blocks_painted", 1);
    }
  }
  if (projectile.typeId !== "minecraft:splash_potion") {
    mc.world.scoreboard.getObjective("ascent_paint:jig_computer.addon_stats")?.addScore("ascent_paint:paintballs_thrown", 1);
    projectile.remove();
  }
}
function getColorFromItem(itemTypeId) {
  return Object.keys(paintbrushColors).find(
    (color) => paintbrushColors[color].includes(itemTypeId)
  );
}
function getHitBlock(player) {
  const hit = player.getBlockFromViewDirection({ maxDistance: 9 });
  return hit ? hit.block : null;
}
function tryReplaceBlock(player, hitBlock, color) {
  const currentColor = extractColor(hitBlock.typeId);
  if (currentColor === color) {
    return false;
  }
  let newBlockType;
  if (currentColor) {
    const baseBlockType = hitBlock.typeId.substring(0, hitBlock.typeId.length - `_${currentColor}`.length);
    newBlockType = `${baseBlockType}_${color}`;
  } else {
    for (const x of BlockMapping) {
      if (hitBlock.matches(x.from, x.fromData)) {
        if (x.to.includes("minecraft:")) {
          newBlockType = `minecraft:${color}_${x.to.split(":")[1]}`;
          if (x.from.includes(color))
            return;
        } else {
          newBlockType = `${x.to}_${color}`;
        }
        break;
      }
    }
  }
  if (!newBlockType) {
    sendDeveloperMessage(player, `\xA74\xA7l[!]\xA7c\xA7l Paint Error:\xA7r\xA77 Block ${hitBlock.typeId} cannot be painted.\xA7r`, "note.bassattack");
    return false;
  }
  return replaceBlock(player, hitBlock, newBlockType, hitBlock.typeId);
}
function tryRevertBlock(player, hitBlock) {
  const currentColor = extractColor(hitBlock.typeId);
  if (!currentColor) {
    const originalBlock2 = VanillaRevertMapping.find((x) => x.from === hitBlock.typeId);
    if (originalBlock2 !== void 0) {
      return replaceBlock(player, hitBlock, originalBlock2.to, hitBlock.typeId);
    }
    sendDeveloperMessage(player, `\xA74\xA7l[!]\xA7c\xA7l Paint Error:\xA7r\xA77 Block ${hitBlock.typeId} is not a painted block.\xA7r`, "note.bassattack");
    return false;
  }
  const originalBlock = BlockMapping.find((mapping) => mapping.to === hitBlock.typeId.replace(`_${currentColor}`, ""));
  if (!originalBlock) {
    sendDeveloperMessage(player, `\xA74\xA7l[!]\xA7c\xA7l Paint Error:\xA7r\xA77 Original block type for ${hitBlock.typeId} not found.\xA7r`, "note.bassattack");
    return false;
  }
  return replaceBlock(player, hitBlock, originalBlock.from, hitBlock.typeId, originalBlock.fromData);
}
function replaceBlock(player, block, newBlockType, oldBlockType, newBlockData = void 0) {
  try {
    let newPermutation = mc.BlockPermutation.resolve(newBlockType, newBlockData);
    for (let [key, value] of Object.entries(block.permutation.getAllStates())) {
      let newKey = stateMapping.find((x) => x.from === key);
      if (newKey && newPermutation.getState(newKey.to) !== void 0) {
        newPermutation = newPermutation.withState(newKey.to, newKey.toValue[newKey.fromValue.indexOf(value)]);
      }
      newKey = stateMapping.find((x) => x.to === key);
      if (newKey && newPermutation.getState(newKey.from) !== void 0) {
        newPermutation = newPermutation.withState(newKey.from, newKey.fromValue[newKey.toValue.indexOf(value)]);
      }
    }
    block.setPermutation(mc.BlockPermutation.resolve(newBlockType, newPermutation.getAllStates()));
    return true;
  } catch (error) {
    sendDeveloperMessage(player, `\xA74\xA7l[!]\xA7c\xA7l Paint Error:\xA7r\xA77 Failed to replace ${oldBlockType} with ${newBlockType}.\xA7r ${error}`, "note.bassattack");
    return false;
  }
}
function handleToolDurability(player, itemUse) {
  if (player.getGameMode() === "creative") {
    return;
  }
  const durabilityComponent = itemUse.getComponent("durability");
  if (!durabilityComponent) {
    return sendDeveloperMessage(player, `\xA74\xA7l[!]\xA7c\xA7l Paint Error:\xA7r\xA77 Item ${itemUse.typeId} does not have a durability component.\xA7r`, "note.bassattack");
  }
  const equippableComponent = player.getComponent("equippable");
  if (!equippableComponent) {
    return sendDeveloperMessage(player, `\xA74\xA7l[!]\xA7c\xA7l Paint Error:\xA7r\xA77 Could not get the equippable component for player.\xA7r`, "note.bassattack");
  }
  durabilityComponent.damage += 1;
  const { maxDurability, damage: currentDamage } = durabilityComponent;
  if (currentDamage >= maxDurability) {
    let replacementItem;
    if (itemUse.typeId === "ascent_paint:cleaning_sponge") {
      replacementItem = new ItemStack("minecraft:air", 1);
      mc.world.scoreboard.getObjective("ascent_paint:jig_computer.addon_stats")?.addScore("ascent_paint:cleaning_sponges_used", 1);
    } else {
      const isPremium = Object.values(paintbrushColors).some((colorArray) => colorArray.includes(itemUse.typeId) && itemUse.typeId.startsWith("ascent_paint:premium"));
      if (isPremium) {
        replacementItem = new ItemStack("ascent_paint:empty_premium_paintbrush", 1);
        mc.world.scoreboard.getObjective("ascent_paint:jig_computer.addon_stats")?.addScore("ascent_paint:premium_paintbrushes_emptied", 1);
      } else {
        replacementItem = new ItemStack("ascent_paint:empty_paintbrush", 1);
        mc.world.scoreboard.getObjective("ascent_paint:jig_computer.addon_stats")?.addScore("ascent_paint:paintbrushes_emptied", 1);
      }
    }
    replaceItemInHand(player, equippableComponent, replacementItem, `Failed to replace ${itemUse.typeId} with ${replacementItem.typeId}.`);
    return true;
  } else {
    replaceItemInHand(player, equippableComponent, itemUse, "Failed to update paintbrush durability.");
  }
}
function replaceItemInHand(player, equippableComponent, item, errorMsg) {
  try {
    equippableComponent.setEquipment("Mainhand", item);
  } catch (error) {
    sendDeveloperMessage(player, `\xA74\xA7l[!]\xA7c\xA7l Paint Error:\xA7r\xA77 ${errorMsg}\xA7r`, "note.bassattack");
  }
}
function sendDeveloperMessage(player, message, sound) {
  if (player === void 0)
    return;
  if (player.hasTag("ascent_paint:dev")) {
    player.sendMessage(message);
    player.playSound(sound, { volume: 0.1 });
  }
}
