import { system, world, BlockPermutation } from '@minecraft/server';
import * as LMC from './LvMfConstants.js';

export class FurnitureClass {

    static isRotating = false; // Lock to prevent spamming

	static async furnitureRotate(source, angle = 90, spawn = false) {
        let rotation;
        if (this.isRotating && !spawn) {
			return;
		}
        source.setProperty('lv.mf:rotating', 1);
		this.isRotating = true;
        await this.sleep(1);
        if (spawn) {
            rotation = this.getNormalizedRotation(source);
        } else {
            this.furnitureBlock(source, source.getProperty('lv.mf:rotation'), 'minecraft:air');
            rotation = source.getProperty('lv.mf:rotation') + angle;
        }
        if (rotation < 0) rotation = 270;
        if (rotation >= 360) rotation = 0;
        
		const newLocation = { x: source.location.x, y: source.location.y + 0.3, z: source.location.z };
		const newLocation1 = { x: source.location.x, y: source.location.y, z: source.location.z };
        const newRotation = { x: 0, y: rotation };
        source.teleport(newLocation);
        source.setRotation(newRotation);
        source.setProperty('lv.mf:rotation', rotation);
        await this.sleep(1);
        source.teleport(newLocation1);
        this.furnitureUpdate(source, rotation);
        await this.sleep(1);
        this.furnitureBlock(source, rotation);
        await this.sleep(10);
        source.setProperty('lv.mf:rotating', 0);
		this.isRotating = false;
    }




    static async furnitureUpdate(source, facing, removeBlock = 0) {
        const { x, y, z } = source.location;
        //console.warn(facing)
        let xFront = 0, zFront = 0;
        let xLeftSide = 0, zLeftSide = 0;
        let xRightSide = 0, zRightSide = 0;

        switch (facing) {
            case 0: // North
                zFront = 1;
                xLeftSide = 1;
                xRightSide = -1;
                break;
            case 90: // East
                xFront = -1;
                zLeftSide = 1;
                zRightSide = -1;
                break;
            case 180: // South
                zFront = -1;
                xLeftSide = -1;
                xRightSide = 1;
                break;
            case 270: // West
                xFront = 1;
                zLeftSide = -1;
                zRightSide = 1;
                break;
        }

        const calculateLocation = (xOffset, zOffset) => ({ x: x + xOffset, y, z: z + zOffset });

        const locations = {
            front: calculateLocation(xFront, zFront),
            leftSide: calculateLocation(xLeftSide, zLeftSide),
            rightSide: calculateLocation(xRightSide, zRightSide),
            frontLeftSide: calculateLocation(xLeftSide + xFront, zLeftSide + zFront),
            frontRightSide: calculateLocation(xRightSide + xFront, zRightSide + zFront),
        };

        const testEntityAtLocation = async (location, rotation, isCorner) => {
            let properties = '';
            if (rotation !== undefined || isCorner !== undefined) {
                properties = `,has_property={`;
                if (rotation !== undefined) {
                    properties += `lv.mf:rotation=${rotation}`;
                }
                if (isCorner !== undefined) {
                    properties += (rotation !== undefined ? ',' : '') + `lv.mf:is_corner=${removeBlock}`;
                }
                properties += '}';
            }

            const command = `testfor @e[type=${source.typeId},x=${location.x},y=${y},z=${location.z},r=0.5${properties}]`;
            const result = await source.runCommandAsync(command);
            return result.successCount > 0;
        };

        try {
            const results = await Promise.all([
                testEntityAtLocation(locations.leftSide),
                testEntityAtLocation(locations.rightSide),
                testEntityAtLocation(locations.frontLeftSide, facing >= 270 ? 0 : facing + 90),
                testEntityAtLocation(locations.frontRightSide, facing <= 0 ? 270 : facing - 90),
                testEntityAtLocation(locations.front, facing >= 270 ? 0 : facing + 90),
                testEntityAtLocation(locations.rightSide, facing),
                testEntityAtLocation(locations.front, facing <= 0 ? 270 : facing - 90),
                testEntityAtLocation(locations.leftSide, facing),
                testEntityAtLocation(locations.leftSide, undefined, 1),
                testEntityAtLocation(locations.rightSide, undefined, 1)
            ]);

            if (results[0] && results[2] && !removeBlock) {
                this.furnitureGetEntity(source, locations.leftSide, async(entity) => {
                    try {
                        entity.setProperty('lv.mf:is_corner', 1);
                        this.fillBlock(source, 'minecraft:air', x, y, z, x, y, z);
                        entity.setProperty('lv.mf:rotation', facing);
                        await this.sleep(1);
                        this.fillBlock(source, 'minecraft:barrier', x, y, z, x, y, z);
                    } catch (e) {
                        //console.error('Error setting entity properties:', e);
                    }
                });
            }

            if (results[1] && results[3] && !removeBlock) {
                this.furnitureGetEntity(source, locations.rightSide, async(entity) => {
                    let facing1 = (facing <= 0 ? 270 : facing - 90);
                    try {
                        entity.setProperty('lv.mf:is_corner', 1);
                        this.fillBlock(source, 'minecraft:air', x, y, z, x, y, z);
                        entity.setProperty('lv.mf:rotation', facing1);
                        await this.sleep(1);
                        this.fillBlock(source, 'minecraft:barrier', x, y, z, x, y, z);
                    } catch (e) {
                        //console.error('Error setting entity properties:', e);
                    }
                });
            }

            if (results[4] && results[5] && !removeBlock) {
                source.setProperty('lv.mf:is_corner', 1);
                source.setProperty('lv.mf:rotation', facing);
            }

            if (results[6] && results[7] && !removeBlock) {
                let facing1 = (facing <= 0 ? 270 : facing - 90);
                source.setProperty('lv.mf:is_corner', 1);
                source.setProperty('lv.mf:rotation', facing1);
            }

            if (results[8] && removeBlock === 1) {
                this.furnitureGetEntity(source, locations.leftSide, async(entity) => {
                    try {
                        entity.setProperty('lv.mf:is_corner', 0);
                    } catch (e) {
                        //console.error('Error updating entity properties:', e);
                    }
                });
            }

            if (results[9] && removeBlock === 1) {
                this.furnitureGetEntity(source, locations.rightSide, async(entity) => {
                    try {
                        entity.setProperty('lv.mf:is_corner', 0);
                    } catch (e) {
                        //console.error('Error updating entity properties:', e);
                    }
                });
            }
        } catch (error) {
            //console.error('Error running command:', error);
        }
    }

    static async furnitureGetEntity(source, location, callback) {
        const entities = source.dimension.getEntities({
            location: location,
            closest: 1,
            maxDistance: 1,
            type: source.typeId
        });

        for (let entity of entities) {
            if (callback) {
                await callback(entity);
            }
        }
    }

    static furnitureNameTag(source, name = '§l') {
        const newNameTag = name;
        source.nameTag = newNameTag;
    }

    static furnitureBlock(source, rotation, block = 'minecraft:barrier', permutation = {}) {
        let { x, y, z } = source.location;
        const normalizedRotation = rotation;
        const furniture = source.typeId;
        const barrierBlockArray = LMC.barrierBlocks[furniture];
        //console.warn(normalizedRotation)

        if (!barrierBlockArray) {
            //console.error("Unknown structure type:", furniture);
            return;
        }

        const barrierObj = barrierBlockArray.find(obj => Math.abs(obj.orientation - normalizedRotation) < 1);

        if (barrierObj) {
            const { xOffSet, yOffSet, zOffSet, xPlus, zPlus } = barrierObj;
            let targetX = x + xOffSet;
            const targetY = y + yOffSet;
            let targetZ = z + zOffSet;

            if (targetX < x) [x, targetX] = [targetX, x];
            if (targetZ < z) [z, targetZ] = [targetZ, z];
            this.fillBlock(source, block, (x + xPlus), y, (z + zPlus), targetX, targetY, targetZ, permutation);
        } else {
            //console.error("No matching orientation found for the given structure:", furniture);
        }
    }

    static fillBlock(source, blockPerm, xFrom, yFrom, zFrom, xTo, yTo, zTo, permutation = {}) {
        // Ensure yFrom and yTo are within the specified range
        yFrom = Math.max(-64, yFrom);
        yTo = Math.min(320, yTo);

        for (let i = xFrom; i <= xTo; i++) {
            for (let j = yFrom; j <= yTo; j++) {
                for (let k = zFrom; k <= zTo; k++) {
                    const block = source.dimension.getBlock({ x: i, y: j, z: k });
                    if (block.permutation.matches('minecraft:air') || block.permutation.matches('minecraft:barrier') || block.permutation.matches('minecraft:light_block')) {
                        block.setPermutation(BlockPermutation.resolve(blockPerm, permutation));
                    }
                }
            }
        }
    }

    static async sleep(ticks) {
        return new Promise((resolve) => system.runTimeout(() => resolve(undefined), ticks));
    }

    static getNormalizedRotation(source) {
        let view = source.getRotation();
        let roundedRotation = Math.round(view.y / 90) * 90;
        let normalizedRotation = ((roundedRotation % 360) + 360) % 360;
        return normalizedRotation;
    }
}
