import { system, world, BlockPermutation, ItemStack } from '@minecraft/server';
import { FurnitureClass } from './FurnitureClass.js';
import * as LMC from './LvMfConstants.js';

export class LvMfClass {
	static run() {
		world.afterEvents.playerSpawn.subscribe((event) => {
			const { player, initialSpawn } = event;
			if (!initialSpawn) return;
			const giveBook = system.runInterval(() => {
				try {
					if (!player.hasTag('lv_mf:guide_book') && player.isOnGround) {
						world.structureManager.place('lv_mf:lv_mf.spawn_book', player.dimension, player.location);
						player.addTag('lv_mf:guide_book');
						system.clearRun(giveBook);
					}
				} catch (e) {}
			},1)
			
		});

		world.afterEvents.entitySpawn.subscribe((e) => {
			const { entity, cause } = e;
			if (cause === 'Spawned') {
				if (!LMC.furnitures.includes(entity.typeId)) return;
				FurnitureClass.furnitureRotate(entity, null, true)
				FurnitureClass.furnitureNameTag(entity)
			}
		});

		world.afterEvents.entityHitEntity.subscribe((e) => {
			const { damagingEntity, hitEntity } = e;
			const mainhand = damagingEntity.getComponent('equippable').getEquipment('Mainhand')?.typeId;
			switch (mainhand) {
				case 'lv_mf:paint_brush':
					if (!LMC.canPaint.includes(hitEntity.typeId)) return;
					const currentSkins = hitEntity.getProperty('lv.mf:skins');
					const newSkins = currentSkins <= 0 ? LMC.maxSkins[hitEntity.typeId] : currentSkins - 1;
					hitEntity.setProperty('lv.mf:skins', newSkins);
					hitEntity.runCommand("/playsound lv_mf.paint_brush_use @a");
					break;
				case 'lv_mf:rotate_tool':
					if (!LMC.canRotate.includes(hitEntity.typeId)) return;
					FurnitureClass.furnitureRotate(hitEntity, -90)
					hitEntity.runCommand("/playsound lv_mf.wrench_use @a");
					break;
			}
		});

		system.afterEvents.scriptEventReceive.subscribe(e => {
			const { id, message, sourceEntity } = e;
			const { x, y, z } = sourceEntity.location;
			switch (id) {
				case 'lv.mf:furniture.paint':
					if (!LMC.canPaint.includes(sourceEntity.typeId)) return;
					const currentSkins = sourceEntity.getProperty('lv.mf:skins');
					const newSkins = currentSkins >= LMC.maxSkins[sourceEntity.typeId] ? 0 : currentSkins + 1;
					sourceEntity.setProperty('lv.mf:skins', newSkins);
					sourceEntity.runCommand("/playsound lv_mf.paint_brush_use @a");
					break;
				case 'lv.mf:furniture.rotate':
					if (!LMC.canRotate.includes(sourceEntity.typeId)) return;
					FurnitureClass.furnitureRotate(sourceEntity, 90)
					sourceEntity.runCommand("/playsound lv_mf.wrench_use @a");
					break;
				case 'lv.mf:furniture.remove':
					FurnitureClass.furnitureBlock(sourceEntity, sourceEntity.getProperty('lv.mf:rotation'), 'minecraft:air');
					FurnitureClass.fillBlock(sourceEntity, 'minecraft:air', x, y, z, x, y, z)
					FurnitureClass.furnitureUpdate(sourceEntity, sourceEntity.getProperty('lv.mf:rotation'), 1);
					system.runTimeout(() => {
						const item = new ItemStack(`${sourceEntity.typeId}_spawn_egg`);
						sourceEntity.dimension.spawnItem(item, sourceEntity.location);
						sourceEntity.remove()
					}, 2);
					sourceEntity.runCommand("/playsound lv_mf.hammer_use @a");
					break;
				case 'lv.mf:furniture.activate':
					const active = sourceEntity.getDynamicProperty('activate');
					sourceEntity.runCommand("/playsound lv_mf.toggle_lights @a");
					if (!active) {
						FurnitureClass.fillBlock(sourceEntity, 'minecraft:light_block', x, y, z, x, y, z, { block_light_level: 8 });
						sourceEntity.setDynamicProperty('activate', true);
						sourceEntity.setProperty('lv.mf:is_active', 1);
					} else {
						FurnitureClass.fillBlock(sourceEntity, 'minecraft:air', x, y, z, x, y, z);
						sourceEntity.setDynamicProperty('activate', false);
						sourceEntity.setProperty('lv.mf:is_active', 0);
					}
					break;
				case 'lv.mf:furniture.deactivate':
					break;
			}
		});
	}

	static getRandomInt(min, max) {
		min = Math.ceil(min);
		max = Math.floor(max);
		return Math.floor(Math.random() * (max - min + 1)) + min;
	}

	static getObjective(obj, displayName) {
		const s = world.scoreboard;
		return s.getObjective(obj) ?? s.addObjective(obj, displayName ?? obj);
	}
}
